include <library.scad>

$fn = 64;

module rocket() {
	bell_radii = [3.1, 5.2, 6.5, 7.3, 7.8, 8.0];
	z_fudge = 0.6;

	bell_slice_depth = (unit-plinth_height+z_fudge)/(len(bell_radii)-1);
	translate([0, 0, -unit/2]) {
		rotate_extrude() union() {
			polygon(concat([
				for (i = [0:len(bell_radii)-1]) [bell_radii[i], plinth_height+z_fudge+i*bell_slice_depth]
			], [
				for (i = [len(bell_radii)-1:-1:0]) [bell_radii[i]-1, unit-(unit-(plinth_height+z_fudge+i*bell_slice_depth))*0.7]
			], [
				[0, unit-(unit-plinth_height+z_fudge)*0.7],
				[0, overlap],
				[bell_radii[0], overlap],
			]));
			for (i = [1:len(bell_radii)-2]) {
				translate([bell_radii[i], plinth_height+z_fudge+i*bell_slice_depth+sin(45)]) rotate(45) square(1, center=true);
			}
		}
		rotate(90, [1, 0, 0]) linear_extrude(1) polygon(concat([
				for (i = [0:len(bell_radii)-2]) [bell_radii[i]+0.5, plinth_height+z_fudge+i*bell_slice_depth]
			], [
				[bell_radii[len(bell_radii)-1], unit],
			], [
				for (i = [len(bell_radii)-1:-1:0]) [bell_radii[i]-overlap, plinth_height+z_fudge+i*bell_slice_depth]
			]));
		translate([0, 0, plinth_height-overlap]) cylinder(1.2+overlap, d=slot_separation);
	}
	plinth();
}

module canard() {
	intersection() {
		rotate(90, [1, 0, 0]) linear_extrude(unit, center=true) polygon([
			[-unit*0.05, unit/2],
			[unit*0.05, unit/2],
			[unit*0.1, -unit/2],
			[-unit*0.1, -unit/2],
		]);
		rotate(-90, [0, 1, 0]) linear_extrude(unit, center=true) polygon([
			[-unit/2+plinth_height, unit*0.3],
			[unit*0.4, 0],
			[unit/2, -unit*0.3],
			[-unit/2+plinth_height, -unit*0.2],
		]);
	}
	plinth();
}

module propeller() union() {
	translate([0, 0, unit/8-unit/2]) mirror([0, 0, 1]) linear_extrude(unit/8, twist=-30, scale=0.8) offset(r=unit/32) {
		square([unit, unit/100], center=true);
		square([unit/100, unit], center=true);
	}
	intersection() {
		translate([0, 0, -unit/2]) cylinder(unit/2, d1=unit/8, d2=slot_separation/sin(45));
		translate([-slot_separation/2, -slot_separation/2, -unit/2]) cube([slot_separation, slot_separation, unit]);
	}
	translate([0, 0, plinth_height-unit/2]) mirror([0, 0, 1]) plinth();
}

module solar_panel() difference() {
	depth = unit/16;
	grid_width = 10;
	grid_height = 4;
	margin = unit/32;
	surface_width = unit+grid-plinth_height-slot_separation/2+depth/2-margin*2;
	surface_height = slot_separation-slot_lip_fudge-margin*2;

	translate([0, 0, -slot_separation/2+slot_lip_fudge]) linear_extrude(slot_separation-slot_lip_fudge) for (y = [0, 1]) mirror([0, y, 0]) polygon([
		[-unit/2, slot_separation/2],
		[plinth_height-unit/2, slot_separation/2],
		[plinth_height+slot_separation/2-depth/2-unit/2, depth/2],
		[unit/2+grid, depth/2],
		[unit/2+grid, -depth/2],
		[-unit/2, -depth/2],
	]);
	for (x = [0:grid_width-1]) for (y = [0, 1]) for (z = [0:grid_height-1]) mirror([0, y, 0]) translate([x*surface_width/grid_width-unit/2+plinth_height+slot_separation/2-depth/2+margin, depth*0.4, z*(surface_height/grid_height)-slot_separation/2+slot_lip_fudge+margin]) cube([surface_width/grid_width*0.8, depth, surface_height/grid_height*0.8]);
	vslots_x_negative();
}

rocket();
translate([grid, 0, 0]) canard();
translate([grid*2, 0, 0]) propeller();
translate([grid*3, 0, 0]) solar_panel();
