include <library.scad>

$fn = 64;

module partial_column() difference() {
	translate([-slot_separation/2, -unit/2, slot_lip_fudge-slot_separation/2]) cube([slot_separation, unit, slot_separation-slot_lip_fudge]);
	vslots_y_negative();
	vslots_y_positive();
}

module partial_column_cylindrical() difference() {
	cylinder(unit, d=slot_separation, center=true);
	vslots_z_negative();
	vslots_z_positive();
}

module partial_slant() difference() {
	slot_height = slot_clip_length+slot_lip;
	offset = grid/2;

	translate([0, 0, slot_lip_fudge-slot_separation/2]) linear_extrude(slot_separation-slot_lip_fudge) polygon([
		[-slot_separation/2+offset/2, unit/2],
		[slot_separation/2+offset/2, unit/2],
		[slot_separation/2+offset/2, unit/2-slot_height],
		[slot_separation/2-offset/2, -unit/2+slot_height],
		[slot_separation/2-offset/2, -unit/2],
		[-slot_separation/2-offset/2, -unit/2],
		[-slot_separation/2-offset/2, -unit/2+slot_height],
		[-slot_separation/2+offset/2, unit/2-slot_height],
	]);
	translate([-offset/2, 0, 0]) vslots_y_negative();
	translate([offset/2, 0, 0]) vslots_y_positive();
}

module partial_bend() difference() {
	slot_height = slot_clip_length+slot_lip;

	translate([0, 0, slot_lip_fudge-slot_separation/2]) linear_extrude(slot_separation-slot_lip_fudge) polygon([
		[unit/2-slot_height, slot_separation/2],
		[unit/2, slot_separation/2],
		[unit/2, -slot_separation/2],
		[unit/2-slot_height, -slot_separation/2],
		[slot_separation/2, slot_height-unit/2],
		[slot_separation/2, -unit/2],
		[-slot_separation/2, -unit/2],
		[-slot_separation/2, slot_height-unit/2],
	]);
	vslots_x_positive();
	vslots_y_negative();
}

module partial_curve() difference() {
	translate([0, 0, slot_lip_fudge-slot_separation/2]) linear_extrude(slot_separation-slot_lip_fudge) union() {
		intersection() {
			translate([slot_separation/2, -slot_separation/2]) circle(slot_separation);
			square(slot_separation, center=true);
		}
		translate([-slot_separation/2, -unit/2]) square([slot_separation, slot_clip_length+slot_lip]);
		translate([unit/2-slot_clip_length-slot_lip, -slot_separation/2]) square([slot_clip_length+slot_lip, slot_separation]);
	}
	vslots_x_positive();
	vslots_y_negative();
}

module partial_cap() {
	plinth();
	translate([-slot_separation/2, -slot_separation/2, plinth_height-unit/2]) cube([slot_separation, slot_separation, unit/2-plinth_height]);
}

module partial_cap_bevel() {
	size = slot_separation*sin(45);

	plinth();
	intersection() {
		translate([0, 0, plinth_height-unit/2]) rotate(45, [1, 0, 0]) cube([unit, size, size], center=true);
		translate([0, 0, plinth_height-unit/2]) rotate(45, [0, 1, 0]) cube([size, unit, size], center=true);
		translate([-unit/2, -unit/2, plinth_height-unit/2-overlap]) cube([unit, unit, unit/2]);
	}
}

module partial_cap_conical() {
	plinth();
	intersection() {
		translate([0, 0, plinth_height-overlap-unit/2]) cylinder(unit+overlap-plinth_height, d1=unit, d2=0.2);
		translate([-slot_separation/2, -slot_separation/2, plinth_height-unit/2-overlap]) cube([slot_separation, slot_separation, unit]);
	}
}

module partial_cap_spherical() {
	plinth();
	intersection() {
		translate([0, 0, plinth_height-unit/2]) sphere(d=slot_separation/sin(45));
		translate([-slot_separation/2, -slot_separation/2, plinth_height-unit/2-overlap]) cube([slot_separation, slot_separation, unit]);
	}
}

partial_column();
translate([grid, 0, 0]) partial_column_cylindrical();
translate([grid*2, 0, 0]) partial_slant();
translate([grid*3, 0, 0]) partial_bend();
translate([grid*4, 0, 0]) partial_curve();
translate([grid*5, 0, 0]) partial_cap();
translate([grid*6, 0, 0]) partial_cap_bevel();
translate([grid*7, 0, 0]) partial_cap_conical();
translate([grid*8, 0, 0]) partial_cap_spherical();
